package com.cj.jiaqingjiayi.controller;



import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cj.jiaqingjiayi.common.BaseResponse;
import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.common.ResultUtils;
import com.cj.jiaqingjiayi.contant.UserConstant;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.exception.ThrowUtils;
import com.cj.jiaqingjiayi.model.CommonRequest;
import com.cj.jiaqingjiayi.model.domain.*;
import com.cj.jiaqingjiayi.model.request.order.*;
import com.cj.jiaqingjiayi.model.vo.OrdersVO;
import com.cj.jiaqingjiayi.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 订单相关接口
 **/
@Slf4j
@RestController
@Api(tags = "订单接口")
@RequestMapping("/orders")
public class OrdersController {

    @Resource
    private UserService userService;

    @Resource
    private OrdersService ordersService;

    @Resource
    private BusinessService businessService;

    @Resource
    private OrderItemsService orderItemsService;

    @Resource
    private CommoditiesService commoditiesService;


    /**
     * 创建订单
     *
     * @return 订单id
     */
    @ApiOperation(value = "订单创建接口")
    @PostMapping("/add")
    public BaseResponse<Long> addOrders(@RequestBody OrderAddRequest orderAddRequest, HttpServletRequest request) {
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 提取订单
        Orders orders = new Orders();
        User loginUser = userService.getLoginUser(request);
        BeanUtils.copyProperties(orderAddRequest, orders);
        orders.setUserId(loginUser.getId());
        ordersService.validOrder(orders);
        // 提取订单详情
        List<OrderItemsAddRequest> detailAddRequest = orderAddRequest.getOrderItemsAddRequest();
        List<OrderItems> orderItemsList = detailAddRequest.stream().map(item -> {
            OrderItems orderItems = new OrderItems();
            BeanUtils.copyProperties(item, orderItems);
            orderItemsService.validOrderItems(orderItems);
            Long commoditiesId = orderItems.getCommoditiesId();
            // 根据商品设置价格
            Commodities commodities = commoditiesService.getById(commoditiesId);
            BigDecimal commoditiesPrice = BigDecimal.valueOf(commodities.getCommoditiesPrice());
            orderItems.setPrice(commoditiesPrice);
            double subtotal = commodities.getCommoditiesPrice() * orderItems.getQuantity();
            orderItems.setSubtotal(BigDecimal.valueOf(subtotal));
            return orderItems;
        }).toList();
        // 创建订单
        long orderId = ordersService.addOrder(orders, orderItemsList);
        return ResultUtils.success(orderId, "订单创建成功");
    }

    /**
     * 取消订单
     */
    @ApiOperation(value = "订单取消接口")
    @PostMapping("/cancel")
    public BaseResponse<Boolean> cancelOrder(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        Long orderId = commonRequest.getId();
        if (orderId == null || orderId <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        //
        Orders orders = ordersService.getById(orderId);
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR);
        ThrowUtils.throwIf(orders.getPaymentStatus() != 0, ErrorCode.OPERATION_ERROR, "订单状态错误");
        // 判断是否为自己的订单
        User loginUser = userService.getLoginUser(request);
        if (!orders.getUserId().equals(loginUser.getId())) {
            throw new BusinessException(ErrorCode.NO_AUTH);
        }
        // 修改订单状态
        orders.setPaymentStatus(3);
        boolean update = ordersService.updateById(orders);
        ThrowUtils.throwIf(!update, ErrorCode.SYSTEM_ERROR);
        return ResultUtils.success(true);
    }

    /**
     * 根据 id 获取订单信息（仅管理员）
     */
    @ApiOperation(value = "根据 id 获取订单信息（仅管理员）")
    @GetMapping("/get")
    public BaseResponse<OrdersVO> getOrderVOById(@RequestParam Long id) {
        if (id <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Orders orders = ordersService.getById(id);
        ThrowUtils.throwIf(orders == null, ErrorCode.NOT_FOUND_ERROR);
        return ResultUtils.success(ordersService.getOrdersVO(orders));
    }

    /**
     * 分页获取订单列表（管理员）
     */
    @PostMapping("/list/page")
    @ApiOperation(value = "分页获取订单列表（管理员）")
    public BaseResponse<Page<Orders>> listOrdersByPage(@RequestBody OrderQueryRequest orderQueryRequest, HttpServletRequest request) {
        userService.isAdmin(request);
        long current = orderQueryRequest.getCurrent();
        long size = orderQueryRequest.getPageSize();
        Page<Orders> ordersPage = ordersService.page(new Page<>(current, size),
                ordersService.getQueryWrapper(orderQueryRequest));
        return ResultUtils.success(ordersPage);
    }

    /**
     * 获取我的订单
     */
    @ApiOperation(value = "获取自己的订单")
    @PostMapping("/my/page")
    public BaseResponse<Page<OrdersVO>> getMyOrders(@RequestBody MyOrderQueryRequest myOrderQueryRequest, HttpServletRequest request) {
        if (myOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = myOrderQueryRequest.getCurrent();
        long size = myOrderQueryRequest.getPageSize();
        User loginUser = userService.getLoginUser(request);
        OrderQueryRequest orderQueryRequest = new OrderQueryRequest();
        BeanUtils.copyProperties(myOrderQueryRequest, orderQueryRequest);
        QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
        if (loginUser.getUserRole().equals(UserConstant.BUSINESS_ROLE)) {
            // 商家获取订单
            Business loginBusiness = businessService.getLoginBusiness(request);
            queryWrapper.eq("businessId", loginBusiness.getId());
        } else {
            // 用户获取订单
            queryWrapper.eq("userId", loginUser.getId());
        }
        Page<Orders> ordersPage = ordersService.page(new Page<>(current, size),
                queryWrapper);
        Page<OrdersVO> ordersVOPage = new Page<>(current, size, ordersPage.getTotal());
        List<OrdersVO> ordersVOList = ordersService.getOrdersVO(ordersPage.getRecords());
        ordersVOPage.setRecords(ordersVOList);
        return ResultUtils.success(ordersVOPage);
    }

    /**
     * 订单统计
     */
    @ApiOperation(value = "订单统计")
    @PostMapping("/count")
    public BaseResponse<String> ordersCount(@RequestBody OrderCountRequest orderCountRequest) {
        String type = orderCountRequest.getType();
        Integer state = orderCountRequest.getPaymentStatus();
        Long businessId = orderCountRequest.getBusinessId();
        String startTime = orderCountRequest.getStartTime();
        String endTime = orderCountRequest.getEndTime();

        // 参数校验
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR, "Type不能为空");
        ThrowUtils.throwIf(businessId == null, ErrorCode.PARAMS_ERROR, "BusinessId不能为空");

        // 构建查询条件
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(state != null, "paymentStatus", state);
        queryWrapper.eq("businessId", businessId);
        return getStringBaseResponse(type, startTime, endTime, queryWrapper);
    }

    /**
     * 订单金额统计(商家)
     */
    @ApiOperation(value = "订单金额统计")
    @PostMapping("/count/money")
    //@AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<List<BigDecimal>> ordersCountMoney(@RequestBody OrderCountRequest orderCountRequest) {
        // 获取查询类型
        String type = orderCountRequest.getType();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        OrderQueryRequest orderQueryRequest = new OrderQueryRequest();
        BeanUtils.copyProperties(orderCountRequest, orderQueryRequest);
        List<BigDecimal> moneyCountList = new ArrayList<>();
        if (type.equals("week")) {
            LocalDate today = LocalDate.now();
            DateTimeFormatter startFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 00:00:00");
            DateTimeFormatter endFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 23:59:59");
            // 遍历查询7天数据，放入数组
            for (int i = 0; i <= 6; i++) {
                LocalDate date = today.minusDays(i);
                orderQueryRequest.setStartTime(date.format(startFormatter));
                orderQueryRequest.setEndTime(date.format(endFormatter));
                QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
                List<Orders> ordertList = ordersService.list(queryWrapper);
                BigDecimal money = new BigDecimal("0");
                for (Orders order : ordertList) {
                    money = money.add(order.getTotalPrice());
                }
                moneyCountList.add(money);
            }
        } else {
            QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
            List<Orders> ordertList = ordersService.list(queryWrapper);
            BigDecimal money = new BigDecimal("0");
            for (Orders order : ordertList) {
                money = money.add(order.getTotalPrice());
            }
            moneyCountList.add(money);
        }
        return ResultUtils.success(moneyCountList);
    }

    /**
     * 订单数量统计(商家)
     */
    @ApiOperation(value = "订单数量统计")
    @PostMapping("/count/number")
    //@AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<List<Long>> ordersCountNumber(@RequestBody OrderCountRequest orderCountRequest) {
        // 获取查询类型
        String type = orderCountRequest.getType();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        OrderQueryRequest orderQueryRequest = new OrderQueryRequest();
        BeanUtils.copyProperties(orderCountRequest, orderQueryRequest);
        List<Long> numberCountList = new ArrayList<>();
        if (type.equals("week")) {
            LocalDate today = LocalDate.now();
            DateTimeFormatter startFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 00:00:00");
            DateTimeFormatter endFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd 23:59:59");
            // 遍历查询7天数据，放入数组
            for (int i = 0; i <= 6; i++) {
                LocalDate date = today.minusDays(i);
                orderQueryRequest.setStartTime(date.format(startFormatter));
                orderQueryRequest.setEndTime(date.format(endFormatter));
                QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
                long count = ordersService.count(queryWrapper );
                numberCountList.add(count);
            }
        } else {
            QueryWrapper<Orders> queryWrapper = ordersService.getQueryWrapper(orderQueryRequest);
            long count = ordersService.count(queryWrapper);
            numberCountList.add(count);
        }
        return ResultUtils.success(numberCountList);
    }

    /**
     * 订单统计(web端)
     */
    @ApiOperation(value = "订单统计(web端)")
    @PostMapping("/count/web")
    //@AuthCheck(mustRole = UserConstant.BUSINESS_ROLE)
    public BaseResponse<String> ordersCountByWeb(@RequestBody OrderCountRequest orderCountRequest) {
        String type = orderCountRequest.getType();
        Integer state = orderCountRequest.getPaymentStatus();
        String businessName = orderCountRequest.getBusinessName();
        Long businessState = orderCountRequest.getBusinessState();
        String startTime = orderCountRequest.getStartTime();
        String endTime = orderCountRequest.getEndTime();
        List<Long> businessIdList = new ArrayList<>();
        ThrowUtils.throwIf(StringUtils.isBlank(type), ErrorCode.PARAMS_ERROR);
        if (businessName != null || businessState != null) {
            QueryWrapper<Business> wrapper = new QueryWrapper<>();
            wrapper.eq(StringUtils.isNotBlank(businessName), "businessName", businessName);
            wrapper.eq(businessState != null, "state", businessState);
            List<Business> businessList = businessService.list(wrapper);
            if (CollectionUtils.isEmpty(businessList)) {
                return ResultUtils.success("0");
            }
            businessIdList = businessList.stream().map(Business::getId).collect(Collectors.toList());
        }
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(state != null, "paymentStatus", state);
        queryWrapper.in(!CollectionUtils.isEmpty(businessIdList), "businessId", businessIdList);
        return getStringBaseResponse(type, startTime, endTime, queryWrapper);
    }

    private BaseResponse<String> getStringBaseResponse(String type, String startTime, String endTime, QueryWrapper<Orders> queryWrapper) {
        queryWrapper.ge(StringUtils.isNotBlank(startTime), "createTime", startTime);
        queryWrapper.le(StringUtils.isNotBlank(endTime), "createTime", endTime);
        if (type.equals("money")) {
            List<Orders> ordertList = ordersService.list(queryWrapper);
            BigDecimal money = new BigDecimal("0");
            for (Orders order : ordertList) {
                money = money.add(order.getTotalPrice());
            }
            String strMoney = String.valueOf(money);
            return ResultUtils.success(strMoney);
        } else if (type.equals("number")) {
            long count = ordersService.count(queryWrapper);
            String strNumber = String.valueOf(count);
            return ResultUtils.success(strNumber);
        } else {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "type不正确");
        }
    }

}


