package com.cj.jiaqingjiayi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.cj.jiaqingjiayi.common.ErrorCode;
import com.cj.jiaqingjiayi.contant.CommonConstant;
import com.cj.jiaqingjiayi.exception.BusinessException;
import com.cj.jiaqingjiayi.model.domain.Business;
import com.cj.jiaqingjiayi.model.domain.OrderItems;
import com.cj.jiaqingjiayi.model.domain.Orders;
import com.cj.jiaqingjiayi.model.request.order.OrderQueryRequest;
import com.cj.jiaqingjiayi.model.vo.BusinessVO;
import com.cj.jiaqingjiayi.model.vo.OrderItemsVO;
import com.cj.jiaqingjiayi.model.vo.OrdersVO;
import com.cj.jiaqingjiayi.service.BusinessService;
import com.cj.jiaqingjiayi.service.OrderItemsService;
import com.cj.jiaqingjiayi.service.OrdersService;
import com.cj.jiaqingjiayi.mapper.OrdersMapper;
import com.cj.jiaqingjiayi.utils.RegexUtils;
import com.cj.jiaqingjiayi.utils.SqlUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.naming.Name;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
* @author 高木
* @description 针对表【orders(订单表)】的数据库操作Service实现
* @createDate 2024-11-10 14:22:10
*/
@Service
public class OrdersServiceImpl extends ServiceImpl<OrdersMapper, Orders>
    implements OrdersService{

    @Resource
    private OrderItemsService orderItemsService;

    @Resource
    private BusinessService businessService;


    @Transactional(rollbackFor = Exception.class)
    @Override
    public long addOrder(Orders orders, List<OrderItems> orderItemsList) {
        // 获取当前时间的秒级时间戳（去掉年、月、日的前缀）
        String timePart = String.valueOf(System.currentTimeMillis() / 1000).substring(4);
        // 生成两位随机数
        String randomPart = String.format("%02d", new Random().nextInt(100));
        String orderNumber = timePart + randomPart;
        //设置订单号
        orders.setOrderNumber(orderNumber);

        boolean saveOrder = this.save(orders);
        if (!saveOrder) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "创建订单失败，数据库错误");
        }
        // 创建订单商品
        for (OrderItems orderItems : orderItemsList) {
            // 绑定订单和订单详情
            orderItems.setOrderId(orders.getId());
            boolean saveItems = orderItemsService.save(orderItems);
            if (!saveItems) {
                log.error("创建订单商品失败，订单id：" + orders.getId());
                throw new BusinessException(ErrorCode.OPERATION_ERROR, "订单商品创建失败");
            }
        }
        return orders.getId();
    }

    @Override
    public void validOrder(Orders orders) {
        String userName = orders.getUserName();
        String phone = orders.getPhone();
        Long userId = orders.getUserId();
        Long businessId = orders.getBusinessId();
        BigDecimal totalPrice = orders.getTotalPrice();
        Integer payMethod = orders.getPayMethod();
        if (StringUtils.isAnyBlank(userName, phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        if (StringUtils.isNotBlank(phone) && RegexUtils.isPhoneInvalid(phone)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "手机号格式错误");
        }
        if (ObjectUtils.isEmpty(totalPrice) || totalPrice.floatValue() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "价格错误");
        }
        if (ObjectUtils.anyNull(userId, businessId, payMethod)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "参数不全");
        }
    }

    @Override
    public OrdersVO getOrdersVO(Orders orders) {
        if (orders == null) {
            return null;
        }
        OrdersVO ordersVO = new OrdersVO();
        BeanUtils.copyProperties(orders, ordersVO);
        // 关联查询商家信息
        Long businessId = orders.getBusinessId();
        Business business = null;
        if (businessId != null && businessId > 0) {
            business = businessService.getById(businessId);
        }

        BusinessVO businessVO = businessService.getBusinessVO(business);
        ordersVO.setBusinessVO(businessVO);
        // 关联查询订单详情信息
        Long orderId = orders.getId();
        QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", orderId);
        List<OrderItems> orderItemsList = orderItemsService.list(queryWrapper);
        List<OrderItemsVO> orderItemsVOList = orderItemsService.getOrderItemsVO(orderItemsList);
        ordersVO.setOrderItemsVOList(orderItemsVOList);
        // 关联查询评价状态

        // 设置商品数量
        Integer number = 0;
        for (OrderItemsVO detailsVO : orderItemsVOList) {
            number += detailsVO.getQuantity();
        }
        ordersVO.setNumber(number);
        return ordersVO;
    }

    @Override
    public List<OrdersVO> getOrdersVO(List<Orders> ordersList) {
        if (CollectionUtils.isEmpty(ordersList)) {
            return new ArrayList<>();
        }
        return ordersList.stream().map(this::getOrdersVO).collect(Collectors.toList());
    }

    @Override
    public QueryWrapper<Orders> getQueryWrapper(OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "请求参数为空");
        }
        Long id = orderQueryRequest.getId();
        Long userId = orderQueryRequest.getUserId();
        Long businessId = orderQueryRequest.getBusinessId();
        String orderNumber = orderQueryRequest.getOrderNumber();
        Integer claimStatus = orderQueryRequest.getClaimStatus();
        Integer state = orderQueryRequest.getPaymentStatus();
        String startTime = orderQueryRequest.getStartTime();
        String endTime = orderQueryRequest.getEndTime();
        String sortField = orderQueryRequest.getSortField();
        String sortOrder = orderQueryRequest.getSortOrder();
        QueryWrapper<Orders> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(ObjectUtils.isNotEmpty(id), "id", id);
        queryWrapper.eq(ObjectUtils.isNotEmpty(userId), "userId", userId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(businessId), "businessId", businessId);
        queryWrapper.eq(ObjectUtils.isNotEmpty(orderNumber), "orderNumber", orderNumber);
        queryWrapper.eq(ObjectUtils.isNotEmpty(state), "paymentStatus", state);
        queryWrapper.ge(claimStatus != null, "claimStatus", claimStatus);
        queryWrapper.ge(StringUtils.isNotBlank(startTime), "createTime", startTime);
        queryWrapper.le(StringUtils.isNotBlank(endTime), "createTime", endTime);
        queryWrapper.orderBy(SqlUtils.validSortField(sortField), sortOrder.equals(CommonConstant.SORT_ORDER_ASC),
                sortField);
        return queryWrapper;
    }
}




